@@ -22,7 +22,7 @@
 *
 * \section intro_sec Introduction
 *
-* libqalculate is math libary for expression evaluation with units, variables and functions support and CAS functionality.
+* libqalculate is a math library for expression evaluation with units, variables and functions support and CAS functionality.
 *
 * The main parts of the library is the almighty Calculator class, the MathStructure class for mathematical expressions and classes for objects in an expression,
 * mostly of the class Numbers and sub classes of ExpressionItem.
@@ -35,7 +35,7 @@
 * CALCULATOR->loadLocalDefinitions();
 * cout << CALCULATOR->calculateAndPrint("1 + 1", 2000) << endl;\endcode
 * In the above example, the calculation is terminated after two seconds (2000 ms), if it is not finished before then.
-* Applications using localized numbers should first call Calculalor::unlocalizeExpression() on the expression.
+* Applications using localized numbers should first call Calculator::unlocalizeExpression() on the expression.
 *
 * A less simple application might calculate and output the expression separately.
 * \code
@@ -49,9 +49,9 @@
 * \code
 * PrintOptions po;
 * string result_str = CALCULATOR->print(result, 2000, po);\endcode
-* Alternatively MathStructure::format() followed by MathStructure::print() can be used, whithout the possiblity to specify a time limit.
+* Alternatively MathStructure::format() followed by MathStructure::print() can be used, without the possibility to specify a time limit.
 *
-* Central to the flexiblity of libqalculate is the many options passed to evaluating and display functions with EvaluationOptions and PrintOptions.
+* Central to the flexibility of libqalculate is the many options passed to evaluating and display functions with EvaluationOptions and PrintOptions.
 *
 * \section usage_sec Using the library
 *
@@ -102,13 +102,13 @@ struct PlotParameters {
 	float y_max;
 	/// Maximum x-axis value.
 	float x_max;
-	/// If a logarithimic scale shall be used for the y-axis. Default: false
+	/// If a logarithmic scale shall be used for the y-axis. Default: false
 	bool y_log;
-	/// If a logarithimic scale shall be used for the x-axis. Default: false
+	/// If a logarithmic scale shall be used for the x-axis. Default: false
 	bool x_log;
-	/// Logarithimic base for the y-axis. Default: 10
+	/// Logarithmic base for the y-axis. Default: 10
 	int y_log_base;
-	/// Logarithimic base for the x-axis. Default: 10
+	/// Logarithmic base for the x-axis. Default: 10
 	int x_log_base;
 	/// If  a grid shall be shown in the plot. Default: false
 	bool grid;
@@ -167,9 +167,10 @@ typedef enum {
 #define MESSAGE_STAGE_UNSET			0
 
 ///Message categories
-#define MESSAGE_CATEGORY_NONE		0
-#define MESSAGE_CATEGORY_PARSING	1
-#define MESSAGE_CATEGORY_WIDE_INTERVAL	10
+#define MESSAGE_CATEGORY_NONE				0
+#define MESSAGE_CATEGORY_PARSING			1
+#define MESSAGE_CATEGORY_WIDE_INTERVAL			10
+#define MESSAGE_CATEGORY_IMPLICIT_MULTIPLICATION	11
 
 
 /// A message with information to the user. Primarily used for errors and warnings.
@@ -419,7 +420,7 @@ class Calculator {
 	* @param eo Options for the evaluation and parsing of the expression.
 	* @param[out] parsed_struct NULL or a math structure to fill with the result of the parsing of the expression.
 	* @param[out] to_struct NULL or a math structure to fill with unit expression parsed after "to". If expression does not contain a "to" string, and to_struct is a unit or a symbol (a unit expression string), to_struct will be used instead.
-	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
+	* @param make_to_division If true, the expression after "to" will be interpreted as a unit expression to convert the result to.
 	* @returns true if the calculation was successfully started (and finished if msecs > 0).
 	*/
 	bool calculate(MathStructure *mstruct, std::string str, int msecs, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
@@ -429,7 +430,7 @@ class Calculator {
 	* @param eo Options for the evaluation and parsing of the expression.
 	* @param[out] parsed_struct NULL or a math structure to fill with the result of the parsing of the expression.
 	* @param[out] to_struct NULL or a math structure to fill with unit expression parsed after "to". If expression does not contain a "to" string, and to_struct is a unit or a symbol (a unit expression string), to_struct will be used instead.
-	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
+	* @param make_to_division If true, the expression after "to" will be interpreted as a unit expression to convert the result to.
 	* @returns The result of the calculation.
 	*/
 	MathStructure calculate(std::string str, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
@@ -454,7 +455,7 @@ class Calculator {
 	MathStructure calculate(const MathStructure &mstruct, const EvaluationOptions &eo = default_user_evaluation_options, std::string to_str = "");
 	/** Calculates an expression.and outputs the result to a text string. The expression should be unlocalized first with unlocalizeExpression().
 	*
-	* Unlike other functions for expression evaluation this function handles ending "to"-commmands, in addition to unit conversion, such "to hexadecimal" or to "fractions", similar to the qalc application.
+	* Unlike other functions for expression evaluation this function handles ending "to"-commands, in addition to unit conversion, such "to hexadecimal" or to "fractions", similar to the qalc application.
 	*
 	*
 	* @param str Expression.
@@ -467,6 +468,7 @@ class Calculator {
 	std::string calculateAndPrint(std::string str, int msecs = 10000, const EvaluationOptions &eo = default_user_evaluation_options, const PrintOptions &po = default_print_options);
 	std::string calculateAndPrint(std::string str, int msecs, const EvaluationOptions &eo, const PrintOptions &po, std::string *parsed_expression);
 	std::string calculateAndPrint(std::string str, int msecs, const EvaluationOptions &eo, const PrintOptions &po, AutomaticFractionFormat auto_fraction, AutomaticApproximation auto_approx = AUTOMATIC_APPROXIMATION_OFF, std::string *parsed_expression = NULL, int max_length = -1, bool *result_is_comparison = NULL);
+	std::string calculateAndPrint(std::string str, int msecs, const EvaluationOptions &eo, const PrintOptions &po, AutomaticFractionFormat auto_fraction, AutomaticApproximation auto_approx, std::string *parsed_expression, int max_length, bool *result_is_comparison, bool format, int colorize = 0, int tagtype = TAG_TYPE_HTML);
 	int testCondition(std::string expression);
 	//@}
 
@@ -642,7 +644,7 @@ class Calculator {
 	* @param eo Options for the evaluation and parsing of the expression.
 	* @param[out] parsed_struct NULL or a math structure to fill with the result of the parsing of the expression.
 	* @param[out] to_struct NULL or a math structure to fill with unit expression parsed after "to".
-	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
+	* @param make_to_division If true, the expression after "to" will be interpreted as a unit expression to convert the result to.
 	* @returns true if the calculation was successfully started (and finished if msecs > 0).
 	*/
 	bool RPNStackEnter(std::string str, int msecs, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
@@ -658,7 +660,7 @@ class Calculator {
 	* @param eo Options for the evaluation and parsing of the expression.
 	* @param[out] parsed_struct NULL or a math structure to fill with the result of the parsing of the expression.
 	* @param[out] to_struct NULL or a math structure to fill with unit expression parsed after "to".
-	* @param make_to_division If true, the expression after "to" will be interpreted as a unit epxression to convert the result to.
+	* @param make_to_division If true, the expression after "to" will be interpreted as a unit expression to convert the result to.
 	*/
 	void RPNStackEnter(std::string str, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *parsed_struct = NULL, MathStructure *to_struct = NULL, bool make_to_division = true);
 	bool setRPNRegister(size_t index, MathStructure *mstruct, int msecs, const EvaluationOptions &eo = default_user_evaluation_options);
@@ -719,7 +721,7 @@ class Calculator {
 	bool parseAdd(std::string &str, MathStructure *mstruct, const ParseOptions &po);
 	//@}
 
-	/** @name Functions converting epxressions between units. */
+	/** @name Functions converting expressions between units. */
 	//@{
 	/** Converts to a unit expression.
 	* The converted value is evaluated.
@@ -727,9 +729,10 @@ class Calculator {
 	* @param mstruct The value to convert.
 	* @param composite_ Unit expression.
 	* @param eo Evaluation options.
-	* @param[out] units NULL or a math structure to fill with the parsed unit expression(or set to undefined if no units were found).
+	* @param[out] units NULL or a math structure to fill with the parsed unit expression (or set to undefined if no units were found).
 	* @returns Converted value.
 	*/
+	MathStructure convert(const MathStructure &mstruct, std::string composite_, const EvaluationOptions &eo, MathStructure *units, bool transform_orig, MathStructure *parsed_struct = NULL);
 	MathStructure convert(const MathStructure &mstruct, std::string composite_, const EvaluationOptions &eo = default_user_evaluation_options, MathStructure *units = NULL);
 	/** Converts to a unit.
 	* The converted value is evaluated.
@@ -740,11 +743,12 @@ class Calculator {
 	* @param always_convert ...
 	* @returns Converted value.
 	*/
+	MathStructure convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo, bool always_convert, bool convert_to_mixed_units, bool transform_orig, MathStructure *parsed_struct = NULL);
 	MathStructure convert(const MathStructure &mstruct, Unit *to_unit, const EvaluationOptions &eo = default_user_evaluation_options, bool always_convert = true, bool convert_to_mixed_units = true);
 	MathStructure convert(const MathStructure &mstruct, KnownVariable *to_var, const EvaluationOptions &eo = default_user_evaluation_options);
 	MathStructure convert(double value, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_user_evaluation_options);
 	MathStructure convert(std::string str, Unit *from_unit, Unit *to_unit, int milliseconds, const EvaluationOptions &eo = default_user_evaluation_options);
-	/** Depecated: use convert() */
+	/** Deprecated: use convert() */
 	MathStructure convertTimeOut(std::string str, Unit *from_unit, Unit *to_unit, int milliseconds, const EvaluationOptions &eo = default_user_evaluation_options);
 	MathStructure convert(std::string str, Unit *from_unit, Unit *to_unit, const EvaluationOptions &eo = default_user_evaluation_options);
 	MathStructure convertToBaseUnits(const MathStructure &mstruct, const EvaluationOptions &eo = default_user_evaluation_options);
@@ -821,7 +825,7 @@ class Calculator {
 	*
 	* @param exp10 Base-10 exponent of the value.
 	* @param exp The exponent of the unit.
-	* @param all_prefixes If false, prefixes which is not a multiple of thousand (centi, deci, deka, hekto) will be skipped.
+	* @param all_prefixes If false, prefixes which is not a multiple of thousand (centi, deci, deca, hecto) will be skipped.
 	* @returns A prefix or NULL if the unit should be left without prefix.
 	*/
 	DecimalPrefix *getOptimalDecimalPrefix(int exp10, int exp = 1, bool all_prefixes = true) const;
@@ -829,7 +833,7 @@ class Calculator {
 	*
 	* @param exp10 Base-10 exponent of the value.
 	* @param exp The exponent of the unit.
-	* @param all_prefixes If false, prefixes which is not a multiple of thousand (centi, deci, deka, hekto) will be skipped.
+	* @param all_prefixes If false, prefixes which is not a multiple of thousand (centi, deci, deca, hecto) will be skipped.
 	* @returns A prefix or NULL if the unit should be left without prefix.
 	*/
 	DecimalPrefix *getOptimalDecimalPrefix(const Number &exp10, const Number &exp, bool all_prefixes = true) const;
@@ -897,12 +901,14 @@ class Calculator {
 	void saveFunctionCalled();
 	bool checkSaveFunctionCalled();
 	ExpressionItem *getActiveExpressionItem(std::string name, ExpressionItem *item = NULL);
+	ExpressionItem *getActiveExpressionItem(std::string name, ExpressionItem *item, bool ignore_us);
 	ExpressionItem *getInactiveExpressionItem(std::string name, ExpressionItem *item = NULL);
 	ExpressionItem *getActiveExpressionItem(ExpressionItem *item);
 	ExpressionItem *getExpressionItem(std::string name, ExpressionItem *item = NULL);
 	Unit* getUnit(std::string name_);
 	Unit* getUnitById(int id) const;
 	Unit* getActiveUnit(std::string name_);
+	Unit* getActiveUnit(std::string name_, bool ignore_us);
 	Unit* getCompositeUnit(std::string internal_name_);
 	Unit* getLocalCurrency();
 	void setLocalCurrency(Unit *u);
@@ -927,6 +933,7 @@ class Calculator {
 	Variable* getVariable(std::string name_);
 	Variable* getVariableById(int id) const;
 	Variable* getActiveVariable(std::string name_);
+	Variable* getActiveVariable(std::string name_, bool ignore_us);
 	ExpressionItem *addExpressionItem(ExpressionItem *item, bool force = true);
 	MathFunction* addFunction(MathFunction *f, bool force = true, bool check_names = true);
 	DataSet* addDataSet(DataSet *dc, bool force = true, bool check_names = true);
@@ -935,6 +942,7 @@ class Calculator {
 	MathFunction* getFunction(std::string name_);
 	MathFunction* getFunctionById(int id) const;
 	MathFunction* getActiveFunction(std::string name_);
+	MathFunction* getActiveFunction(std::string name_, bool ignore_us);
 	/** Returns variable for an index (starting at zero). All variables can be traversed by starting at index zero and increasing the index until NULL is returned.
 	*
 	* @param index Index of variable.
@@ -1114,6 +1122,10 @@ class Calculator {
 	int saveUnits(const char *file_name, bool save_global = false);
 	int saveFunctions(const char *file_name, bool save_global = false);
 	int saveDataSets(const char *file_name, bool save_global = false);
+	std::string saveTemporaryDefinitions();
+	void saveVariables(void *xmldoc, bool save_global = false, bool save_only_temp = false);
+	void saveUnits(void *xmldoc, bool save_global = false, bool save_only_temp = false);
+	void saveFunctions(void *xmldoc, bool save_global = false, bool save_only_temp = false);
 	//@}
 
 	/** @name Functions for CSV file import/export. */
@@ -1161,7 +1173,7 @@ class Calculator {
 	bool fetchExchangeRates(int seconds, std::string wget_args);
 	/** Download current exchange rates from the Internet to local disc with default wget arguments.
 	*
-	* @param seconds Maximum time for donwload try
+	* @param seconds Maximum time for download try
 	* @returns true if operation was successful.
 	*/
 	bool fetchExchangeRates(int seconds = 15, int n = -1);
@@ -1190,12 +1202,15 @@ class Calculator {
 	* @returns true if gnuplot was found.
 	*/
 	bool canPlot();
+	MathStructure expressionToPlotVector(std::string expression, const MathStructure &min, const MathStructure &max, int steps, bool separate_complex_part, MathStructure *x_vector = NULL, std::string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
 	MathStructure expressionToPlotVector(std::string expression, const MathStructure &min, const MathStructure &max, int steps, MathStructure *x_vector = NULL, std::string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
 	MathStructure expressionToPlotVector(std::string expression, float min, float max, int steps, MathStructure *x_vector = NULL, std::string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
+	MathStructure expressionToPlotVector(std::string expression, const MathStructure &min, const MathStructure &max, const MathStructure &step, bool separate_complex_part, MathStructure *x_vector = NULL, std::string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
 	MathStructure expressionToPlotVector(std::string expression, const MathStructure &min, const MathStructure &max, const MathStructure &step, MathStructure *x_vector = NULL, std::string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
 	MathStructure expressionToPlotVector(std::string expression, float min, float max, float step, MathStructure *x_vector = NULL, std::string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
 	MathStructure expressionToPlotVector(std::string expression, const MathStructure &x_vector, std::string x_var = "\\x", const ParseOptions &po = default_parse_options, int msecs = 5000);
 	bool plotVectors(PlotParameters *param, const std::vector<MathStructure> &y_vectors, const std::vector<MathStructure> &x_vectors, std::vector<PlotDataParameters*> &pdps, bool persistent = false, int msecs = 5000);
+	void forcePersistentPlot(bool persistent = true);
 	bool invokeGnuplot(std::string commands, std::string commandline_extra = "", bool persistent = false);
 	bool closeGnuplot();
 	bool gnuplotOpen();
@@ -1226,6 +1241,9 @@ class Calculator {
 	void endTemporaryEnableIntervalArithmetic();
 	//@}
 
+	bool usesMatlabStyleMatrices() const;
+	void useMatlabStyleMatrices(bool use_matlab_style_matrices);
+
 	void setCustomInputBase(Number nr);
 	void setCustomOutputBase(Number nr);
 	const Number &customInputBase() const;
